Package: MOFA
Type: Package
Title: Multi-Omics Factor Analysis (MOFA)
Version: 1.6.1
Maintainer: Britta Velten <britta.velten@gmail.com>
Author: Ricard Argelaguet, Britta Velten, Damien Arnol, Florian Buettner, Wolfgang Huber, Oliver Stegle
Date: 2018-06-26
License: LGPL-3 | file LICENSE
Description: Multi-Omics Factor Analysis: an unsupervised framework for the integration of multi-omics data sets.
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: rhdf5, dplyr, reshape2, pheatmap, corrplot, ggplot2,
        ggbeeswarm, methods, scales, GGally, RColorBrewer, cowplot,
        ggrepel, MultiAssayExperiment, Biobase, doParallel, foreach,
        reticulate, grDevices, stats, utils
Suggests: knitr, MOFAdata
biocViews: DimensionReduction, Bayesian, Visualization
VignetteBuilder: knitr
LazyData: false
NeedsCompilation: yes
RoxygenNote: 6.1.1
SystemRequirements: Python (>=2.7.0), numpy, pandas, h5py, scipy,
        sklearn, mofapy
PackageStatus: Deprecated
git_url: https://git.bioconductor.org/packages/MOFA
git_branch: RELEASE_3_12
git_last_commit: d50ed60
git_last_commit_date: 2020-10-28
Date/Publication: 2020-10-28
Packaged: 2020-10-29 01:06:16 UTC; biocbuild
