
##################
## Factor Names ##
##################

#' @title factorNames: set and retrieve factor names
#' @name factorNames
#' @rdname factorNames
#' @export
setGeneric("factorNames", function(object) {standardGeneric("factorNames")})

#' @name factorNames
#' @rdname factorNames
#' @aliases factorNames<-
#' @export
setGeneric("factorNames<-", function(object, value) {standardGeneric("factorNames<-")})


##################
## Sample Names ##
##################

#' @title sampleNames: set and retrieve sample names
#' @name sampleNames
#' @rdname sampleNames
#' @export
setGeneric("sampleNames", function(object) {standardGeneric("sampleNames")})

#' @name sampleNames
#' @rdname sampleNames
#' @aliases sampleNames<-
#' @export
setGeneric("sampleNames<-", function(object, value) {standardGeneric("sampleNames<-")})

###################
## Feature Names ##
###################

#' @title featureNames: set and retrieve feature names
#' @name featureNames
#' @rdname featureNames
#' @export
setGeneric("featureNames", function(object) {standardGeneric("featureNames")})

#' @name featureNames
#' @rdname featureNames
#' @aliases featureNames<-
#' @export
setGeneric("featureNames<-", function(object, value) {standardGeneric("featureNames<-")})

################
## View Names ##
################

#' @title viewNames: set and retrieve view names
#' @name viewNames
#' @rdname viewNames
#' @export
setGeneric("viewNames", function(object) {standardGeneric("viewNames")})

#' @name viewNames
#' @rdname viewNames
#' @aliases viewNames<-
#' @export
setGeneric("viewNames<-", function(object, value) {standardGeneric("viewNames<-")})



################
## Dimensions ##
################

#' @title Dimensions: set and retrieve dimensions
#' @name Dimensions
#' @rdname Dimensions
#' @export
setGeneric("Dimensions", function(object) {standardGeneric("Dimensions")})

#' @name Dimensions
#' @rdname Dimensions
#' @aliases Dimensions<-
#' @export
setGeneric("Dimensions<-", function(object, value) {standardGeneric("Dimensions<-")})


################
## Train Data ##
################

#' @title TrainData: set and retrieve training data
#' @name TrainData
#' @rdname TrainData
#' @export
setGeneric("TrainData", function(object) {standardGeneric("TrainData")})

#' @name TrainData
#' @rdname TrainData
#' @aliases TrainData<-
#' @export
setGeneric("TrainData<-", function(object, value) {standardGeneric("TrainData<-")})


################
## Imputed Data ##
################

#' @title ImputedData: set and retrieve imputed data
#' @name ImputedData
#' @rdname ImputedData
#' @export
setGeneric("ImputedData", function(object) {standardGeneric("ImputedData")})

#' @name ImputedData
#' @rdname ImputedData
#' @aliases ImputedData<-
#' @export
setGeneric("ImputedData<-", function(object, value) {standardGeneric("ImputedData<-")})

################
## Expectations ##
################

#' @title Expectations: set and retrieve expectations of model components
#' @name Expectations
#' @rdname Expectations
#' @export
setGeneric("Expectations", function(object) {standardGeneric("Expectations")})

#' @name Expectations
#' @rdname Expectations
#' @aliases Expectations<-
#' @export
setGeneric("Expectations<-", function(object, value) {standardGeneric("Expectations<-")})

################
## InputData ##
################

#' @title InputData: set and retrieve input data
#' @name InputData
#' @rdname InputData
#' @export
setGeneric("InputData", function(object) {standardGeneric("InputData")})

#' @name InputData
#' @rdname InputData
#' @aliases InputData<-
#' @export
setGeneric("InputData<-", function(object, value) {standardGeneric("InputData<-")})

################
## TrainStats ##
################

#' @title TrainStats: set and retrieve training statistcs
#' @name TrainStats
#' @rdname TrainStats
#' @export
setGeneric("TrainStats", function(object) {standardGeneric("TrainStats")})

#' @name TrainStats
#' @rdname TrainStats
#' @aliases TrainStats<-
#' @export
setGeneric("TrainStats<-", function(object, value) {standardGeneric("TrainStats<-")})

################
## ModelOptions ##
################

#' @title ModelOptions: set and retrieve model options
#' @name ModelOptions
#' @rdname ModelOptions
#' @export
setGeneric("ModelOptions", function(object) {standardGeneric("ModelOptions")})

#' @name ModelOptions
#' @rdname ModelOptions
#' @aliases ModelOptions<-
#' @export
setGeneric("ModelOptions<-", function(object, value) {standardGeneric("ModelOptions<-")})

################
## TrainOptions ##
################

#' @title TrainOptions: set and retrieve training options
#' @name TrainOptions
#' @rdname TrainOptions
#' @export
setGeneric("TrainOptions", function(object) {standardGeneric("TrainOptions")})

#' @name TrainOptions
#' @rdname TrainOptions
#' @aliases TrainOptions<-
#' @export
setGeneric("TrainOptions<-", function(object, value) {standardGeneric("TrainOptions<-")})


################
## DataOptions ##
################

#' @title DataOptions: set and retrieve data options
#' @name DataOptions
#' @rdname DataOptions
#' @export
setGeneric("DataOptions", function(object) {standardGeneric("DataOptions")})

#' @name DataOptions
#' @rdname DataOptions
#' @aliases DataOptions<-
#' @export
setGeneric("DataOptions<-", function(object, value) {standardGeneric("DataOptions<-")})

################
## Status ##
################

#' @title Status: set and retrieve training status
#' @name Status
#' @rdname Status
#' @export
setGeneric("Status", function(object) {standardGeneric("Status")})

#' @name Status
#' @rdname Status
#' @aliases Status<-
#' @export
setGeneric("Status<-", function(object, value) {standardGeneric("Status<-")})

################
## FeatureIntercepts ##
################

#' @title FeatureIntercepts: set and retrieve feature intercepts
#' @name FeatureIntercepts
#' @rdname FeatureIntercepts
#' @export
setGeneric("FeatureIntercepts", function(object) {standardGeneric("FeatureIntercepts")})

#' @name FeatureIntercepts
#' @rdname FeatureIntercepts
#' @aliases FeatureIntercepts<-
#' @export
setGeneric("FeatureIntercepts<-", function(object, value) {standardGeneric("FeatureIntercepts<-")})
